<?php
namespace App\HttpController\Base;
use App\Key\K;
use EasySwoole\Component\Context\ContextManager;
use EasySwoole\Http\AbstractInterface\Controller;
use EasySwoole\Http\Message\Status;

class Base extends Controller{
    protected function jsonSuccess($data, $statusCode = 200, $msg = '')
    {
        if (!$this->response()->isEndResponse()) {
            $this->response()->write(json_encode(['data' => $data, 'status' => $statusCode, 'code' => $statusCode, 'message' => $msg], JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
            $this->response()->withHeader('Content-type', 'application/json;charset=utf-8');
            $this->response()->withStatus($statusCode);
            return true;
        } else {
            return false;
        }
    }
    protected function getRequestData()
    {
        $data = $this->request()->getSwooleRequest()->rawContent();
        $data = json_decode($data, true);
        if ($data) return $data;
        return $this->request()->getParsedBody();
    }
    protected function jsonError($msg = '', $statusCode = 404)
    {
        $statusCode = intval($statusCode);
        if (!$this->response()->isEndResponse()) {
            $this->response()->write(json_encode(['status' => $statusCode, 'code' => $statusCode, 'message' => $msg], JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
            $this->response()->withHeader('Content-type', 'application/json;charset=utf-8');
            $this->response()->withStatus(200);
            return true;
        } else {
            return false;
        }
    }

    protected function onRequest(?string $action): ?bool
    {
        parent::onRequest($action);
//        if (!current($this->request()->getHeader('bigemap'))&&!$this->request()->getQueryParam('bigemap')){
//            $this->writeJson(500, 'bigemap-server error');
//            return false;
//        }
        try {
//            ContextManager::getInstance()->set(K::CONTEXT_CACHE_IP, $this->ip);
            $queryData = $this->getRequestData();
            ContextManager::getInstance()->set(K::CONTEXT_CACHE_GET_DATA, $this->request()->getQueryParams());
            ContextManager::getInstance()->set(K::CONTEXT_CACHE_REQUEST_METHOD, $this->request()->getMethod());
            ContextManager::getInstance()->set(K::CONTEXT_CACHE_POST_DATA, $queryData);
            ContextManager::getInstance()->set(K::CONTEXT_CACHE_PUT_DATA, $queryData);
        }catch (\Throwable $throwable){
            echo 'base context error', $throwable->getMessage(), PHP_EOL;
        }
        $response = $this->response();
        $response->withHeader('Access-Control-Allow-Origin', '*');//允许所有跨域
        $response->withHeader('Access-Control-Allow-Methods', 'GET,POST,PUT,DELETE,OPTIONS');
        $response->withHeader('Access-Control-Allow-Credentials', 'true');
        $response->withHeader('Access-Control-Allow-Headers', '*');
        if ($this->request()->getMethod() === 'OPTIONS') {
            $response->withStatus(Status::CODE_OK);
            return false;
        }
        return true;
    }
}
